@@ -2,6 +2,10 @@
 pub const MACRO_START_CHAR: char = '$';
 pub const ESCAPE_CHAR: char ='\\';
 pub const MAIN_CALLER: &str = "@MAIN@";
+#[cfg(feature = "debug")]
+pub const DIFF_SOURCE_FILE : &str = "diff.src";
+#[cfg(feature = "debug")]
+pub const DIFF_OUT_FILE : &str = "diff.out";
 
 // Platform specific consts
 #[cfg(windows)]
